@@ -78,8 +78,30 @@ class DldFlashProcessorExpress(DldProcessor):
                 self.DATA_PARQUET_DIR = Path(parquet_dir)
 
             if not self.DATA_PARQUET_DIR.exists():
-                os.mkdir(self.DATA_PARQUET_DIR)    
-            
+                os.mkdir(self.DATA_PARQUET_DIR)   
+         
+    @property
+    def removeChannels(self, channel_names):
+        """Removes the unnecessary channels from the available channels
+        using list of channels to remove"""
+        for channels in channel_names:
+            self.all_channels.pop(channels, None)
+        self.channels = self.availableChannels # Set all channels, exluding pulseId as default
+        return self.channels
+
+    @property
+    def addChannels(self, channel_dict):
+        """Add new channels using a dict format defined by:
+        "channel_name": {
+            "format": "per_pulse" | "per_train" | "per_electron",
+            "group_name": "channel_group_path",
+            "slice": ":"
+        }
+        """
+        self.all_channels.update(channel_dict)
+        self.channels = self.availableChannels # Set all channels, exluding pulseId as default
+        return self.channels
+
     @property
     def availableChannels(self):
         """Returns the channel names that are available for use, 
@@ -243,9 +265,9 @@ class DldFlashProcessorExpress(DldProcessor):
                     .set_index(self.index_per_pulse))
 
         elif channel_dict["format"] == "per_train":
-                return (Series((np_array[i] for i in train_id.index), name=channel)
-                    .to_frame()
-                    .set_index(train_id))
+            return (Series((np_array[i] for i in train_id.index), name=channel)
+                .to_frame()
+                .set_index(train_id))
         
         else:
             raise ValueError(channel + "has an undefined format. Available formats are \
@@ -308,28 +330,29 @@ class DldFlashProcessorExpress(DldProcessor):
             self.failed_str.append(f'{prq}: {e}')
             self.prq_names.remove(prq)
             
-    def fill_na(self):
+    def fillNA(self):
         """Routine to fill the NaN values with intrafile forward filling. """
-        # First use forward filling method to fill each file's pulse resolved channels.
+        # First use forward filling method to fill each file's pulse and train resolved channels.
+        channels = self.channelsPerPulse + self.channelsPerTrain
         for i in range(len(self.dfs)):
-            self.dfs[i][self.channelsPerPulse] = self.dfs[i][self.channelsPerPulse].fillna(method="ffill")
+            self.dfs[i][channels] = self.dfs[i][channels].fillna(method="ffill")
         
         # This loop forward fills between the consective files. 
         # The first run file will have NaNs, unless another run before it has been defined.
         for i in range(1, len(self.dfs)):
-            subset = self.dfs[i][self.channelsPerPulse] # Take only pulse channels
+            subset = self.dfs[i][channels] # Take only pulse channels
             is_null = subset.loc[0].isnull().values.compute() # Find which column(s) contain NaNs.
             # Statement executed if there is more than one NaN value in the first row from all columns
             if is_null.sum() > 0: 
                 # Select channel names with only NaNs
-                channels_to_overwrite = list(compress(self.channelsPerPulse, is_null[0]))
+                channels_to_overwrite = list(compress(channels, is_null[0]))
                 # Get the values for those channels from previous file
-                values = self.dfs[i-1][self.channelsPerPulse].tail(1).values[0]
+                values = self.dfs[i-1][channels].tail(1).values[0]
                 # Fill all NaNs by those values
                 subset[channels_to_overwrite] = subset[channels_to_overwrite].fillna(
                                                                 dict(zip(channels_to_overwrite, values)))
                 # Overwrite the dataframes with filled dataframes
-                self.dfs[i][self.channelsPerPulse] = subset
+                self.dfs[i][channels] = subset
 
     def readData(self, runs=None, ignore_missing_runs=False, settings=None, channels=None,
              beamtime_dir=None, parquet_path=None, beamtime_id=None, year=None,
@@ -421,7 +444,7 @@ class DldFlashProcessorExpress(DldProcessor):
         else:
             print(f'Loading {len(self.prq_names)} dataframes. Failed reading {len(all_files)-len(self.prq_names)} files.')  
             self.dfs = [dd.read_parquet(fn) for fn in self.prq_names]
-            self.fill_na()
+            self.fillNA()
             df = dd.concat(self.dfs)
             df_electron = df.dropna(subset=self.channelsPerElectron)
             pulse_columns = ['trainId','pulseId','electronId'] + self.channelsPerPulse
